@@ -73,7 +73,7 @@ impl fmt::Display for PushState {
         }
         write!(
             f,
-            "> BOOL  : {}\n> CODE  : {}\n> EXEC  : {}\n> FLOAT : {}\n> INT   : {}\n> BVEC  : {}\n> FVEC  : {}\n> IVEC  : {}\n> NAME  : {}\n> IDS   : {}\n",
+            "> BOOL  : \n{}\n> CODE  : \n{}\n> EXEC  : \n{}\n> FLOAT : \n{}\n> INT   : \n{}\n> BVEC  : \n{}\n> FVEC  : \n{}\n> IVEC  : \n{}\n> NAME  : \n{}\n> IDS   : \n{}\n",
             self.bool_stack.to_string(),
             self.code_stack.to_string(),
             self.exec_stack.to_string(),
@@ -102,6 +102,6 @@ mod tests {
         test_state
             .name_bindings
             .insert("Var1".to_string(), Item::bool(true));
-        assert_eq!(test_state.to_string(), "> BOOL  : \n> CODE  : \n> EXEC  : \n> FLOAT : \n> INT   : \n> BVEC  : \n> FVEC  : \n> IVEC  : \n> NAME  : \n> IDS   : Var1 => Literal(true);\n Var2 => InstructionMeta(INTVECTOR.BOOLINDEX);\n \n")
+        assert_eq!(test_state.to_string(), "> BOOL  : \n\n> CODE  : \n\n> EXEC  : \n\n> FLOAT : \n\n> INT   : \n\n> BVEC  : \n\n> FVEC  : \n\n> IVEC  : \n\n> NAME  : \n\n> IDS   : \nVar1 => Literal(true);\n Var2 => InstructionMeta(INTVECTOR.BOOLINDEX);\n \n")
     }
 }
